#ifndef BUFFER_H_
#define BUFFER_H_

#include <iostream>
#include <cmath>
#include <cstdio>
#include <sstream>
#include <string>
#include <vector>


#define M_PI  3.14159265358979323846 /* pi */


#define ALL 0
#define MANDELBROT 1
#define BUDDHABROT 2
#define BUDDHABULB 3


struct Cur{
  float x;  // X position of cursur
  float y;  // Y position of cursur
  float s;  // Scale of cursur
  float r;  // Rotation of cursur
};

struct Coords{
  float x;
  float y;
};

struct Pixel{
  int x;
  int y;
};


class Buffer{
private:
  std::vector<std::vector<int>> buffer;   // Buffer array
  Cur cur;                // Viewing cursur
  Cur start;
  Cur end;
  float ratio;            // Ratio of image width to hight
  int itterations;        // Number of itterations for render (defult 255)
  float contrast;         // Log contrast of render output (defult 2)
  int maxValue;           // Maximum value of array to normalise
  unsigned char fractal;  // Fractal type
public:
  int width;  // Width of buffer
  int height; // Height of buffer
  int samples; // Rendering subresolution (defult 2)

  Buffer(int _width, int _height);    // Constructor
  void save(std::string _filename);   // Save buffer to PPM image
  void clear();                       // Clear buffer
  std::string stats();                // Print buffer stats to console

  int getMax();                             // Return max value in buffer
  void setItterations(int _itterations);    // Set number of render itterations
  void setSamples(int _subres);              // Set subresolution of render
  void setContrast(float _contrast);        // Set contast of output image
  void setFractal(unsigned char _fractal);  // set type of fractal to render

  void setCursor(float _x, float _y, float _s, float _r);
  void setStart(float _x, float _y, float _s, float _r);
  void setEnd(float _x, float _y, float _s, float _r);
  void cursorUpdate(float _f);
  Coords translate(int _x, int _y);
  Pixel translate(float _x, float _y);
  float map(float _v, float _a1, float _a2, float _b1, float _b2);

  void render(int _x, int _y);
};


#endif
